/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is AddressbooksSynchronizer.
 *
 * The Initial Developer of the Original Code is Gnter Gersdorf.
 * Portions created by the Initial Developer are Copyright (C) 2006
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *  Gnter Gersdorf <G.Gersdorf@ggbs.de>
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

// from "chrome://messenger/content/addressbook/abCommon.js"
//   this file cannot be included in .xul!
const kPersonalAddressbookURI = "moz-abmdbdirectory://abook.mab";
const kCollectedAddressbookURI = "moz-abmdbdirectory://history.mab";


var delFiles=new Array;

var pref = Components.classes["@mozilla.org/preferences-service;1"].getService(Components.interfaces.nsIPrefBranch);
var rdf = Components.classes["@mozilla.org/rdf/rdf-service;1"].getService(Components.interfaces.nsIRDFService);
var directory = rdf.GetResource("moz-abdirectory://").QueryInterface(Components.interfaces.nsIAbDirectory);

function readMabData(mabFile, thunderbird)
{
  if (thunderbird) {
    var directoryService =  Components.classes['@mozilla.org/file/directory_service;1'].
      getService(Components.interfaces.nsIProperties);
    var profileFolder = directoryService.get('ProfD',  Components.interfaces.nsIFile);
    var profileDir = profileFolder.QueryInterface(Components.interfaces.nsILocalFile);
    var nsIMabFile = profileDir.clone().QueryInterface(Components.interfaces.nsILocalFile);
    nsIMabFile.appendRelativePath( mabFile );
  } else
    nsIMabFile=mabFile;
  if (!nsIMabFile.exists()) return '';

  var fs = Components.classes['@mozilla.org/network/file-input-stream;1']
          .createInstance(Components.interfaces.nsIFileInputStream);
  var ss = Components.classes['@mozilla.org/scriptableinputstream;1']
          .createInstance(Components.interfaces.nsIScriptableInputStream);
  try{
    fs.init(nsIMabFile, 0x01, 0000, false);
    ss.init(fs);
    var str = ss.read(-1);
    ss.close();
    fs.close();
    return str;
  } catch(e){ alert('Read file:\n'+e); }
  return "";
}

function writeMabData(mabFile, thunderbird, abData)
{
  if (thunderbird) {
    var directoryService =  Components.classes['@mozilla.org/file/directory_service;1'].
      getService(Components.interfaces.nsIProperties);
    var profileFolder = directoryService.get('ProfD',  Components.interfaces.nsIFile);
    var profileDir = profileFolder.QueryInterface(Components.interfaces.nsILocalFile);
    var nsIMabFile = profileDir.clone().QueryInterface(Components.interfaces.nsILocalFile);
    nsIMabFile.appendRelativePath( mabFile );
  } else
    nsIMabFile=mabFile;

  try {
    //if (nsIMabFile.exists()) nsIMabFile.remove(false);
    var strm = Components.classes["@mozilla.org/network/file-output-stream;1"].
      createInstance(Components.interfaces.nsIFileOutputStream);
    strm.QueryInterface(Components.interfaces.nsIOutputStream);
    strm.QueryInterface(Components.interfaces.nsISeekableStream);
    strm.init( nsIMabFile, 0x20|0x02|0x08, 0600, 0 );
    strm.write( abData, abData.length );
    strm.flush();
    strm.close();
  } catch(e) { alert('File write:\n'+e); }
  return nsIMabFile;
}

function saveMabFile(mabName, mabFile, directory, filename)
{
    var directoryService =  Components.classes['@mozilla.org/file/directory_service;1'].
      getService(Components.interfaces.nsIProperties);
    var profileFolder = directoryService.get('ProfD',  Components.interfaces.nsIFile);
    var profileDir = profileFolder.QueryInterface(Components.interfaces.nsILocalFile);
    var localMabFile = profileDir.clone().QueryInterface(Components.interfaces.nsILocalFile);
    localMabFile.appendRelativePath( mabFile );

    if (!filename)
      filename = mabName.replace( /\W/g, "_" )+".mab";
    try {
      var file = Components.classes["@mozilla.org/file/local;1"].
        createInstance(Components.interfaces.nsILocalFile);
      file.initWithFile( directory );
      file.appendRelativePath( filename );
//TODO: eventuell umbenennen statt lschen
      if (file.exists()) { file.remove(true); }
      localMabFile.copyTo( directory, filename );
        //creates directory, if not existing!
    } catch(e) { return false; }
    return true;
} //

function loadMabFile(mabName, mabFile, directory, filename)
{
    if (!filename) { // sync load
      filename = mabName.replace( /\W/g, "_" )+".mab";
      var file = Components.classes["@mozilla.org/file/local;1"].
        createInstance(Components.interfaces.nsILocalFile);
      file.initWithFile( directory );
      file.appendRelativePath( filename );
    } else {  // single load
      var file = Components.classes["@mozilla.org/file/local;1"].
        createInstance(Components.interfaces.nsILocalFile);
      file.initWithPath( filename );
    }
    if (!file.exists()) {
      alert(strings.getString('nofile'));
      return false;
    }

    var abData=readMabData(file, false);
    if (!abData) {
      alert(strings.getString('readerror'));
      return false;
    }
    if (replaceAddressBook(mabName, mabFile, abData))
      return false;
    else
      return true;
}


function syncedAddressbooks()
{
  // get addressbooks to be synced, always ignore history and pab
  var mabFiles=new Array();
  var cn = directory.childNodes;
  var aBook = cn.getNext();
  while( aBook != null )
  {
      var cur = aBook.QueryInterface(Components.interfaces.nsIAbDirectory);
      var file = cur.directoryProperties.fileName;
      var dirType = cur.directoryProperties.dirType;
      var sync=0;
      var prefName = cur.directoryProperties.prefName;
      try { sync = pref.getBoolPref( prefName+".extensions.addressbookssync" ); } catch(e) { sync=0; }
      if (//uri!=kPersonalAddressbookURI &&            // dont sync pab
          //uri!=kCollectedAddressbookURI &&          // dont sync history
          dirType==2 &&                   // only normal addressbooks
          sync ) {
        mabFiles.push(file);
      }
      try { aBook = cn.getNext(); } catch(e) { break; }
  } // while
  return mabFiles;
}

function getAddressBook( name )
{
    var cn = directory.childNodes;
    while( cn.hasMoreElements() )
    {
      var cur = cn.getNext().QueryInterface(Components.interfaces.nsIAbDirectory);
      if( cur.directoryProperties.fileName == name || cur.directoryProperties.description == name)
          return cur;
    } // while
    return null;
} // getAddressBook

function createAddressBook( name, fileName, syncstate )
{
  try {
    var abProps = Components.classes["@mozilla.org/addressbook/properties;1"].
      createInstance(Components.interfaces.nsIAbDirectoryProperties);
    abProps.description = name;
    abProps.dirType = 2;
    abProps.fileName = fileName;
    directory.createNewDirectory( abProps );
      //for pab and history, this will create a second addressbook, because we
      //were not able to delete it. But the second addressbook ist not available
      //via getAddressBook (see below)
    var cur=getAddressBook(fileName);
    pref.setBoolPref( cur.directoryProperties.prefName+".extensions.addressbookssync", syncstate );
  } catch (e) { /*alert('createAddressBook('+name+','+fileName+')\n'+e);*/ }
} // createAddressBook

function deleteAddressBook( name, fileName )
{
  // Move focus away from the to be deleted addressbook
    // this has to be done in the AddressBook window :-((
//  try {  SetAbView(kPersonalAddressbookURI, false);} catch(e) { alert(e);}

  try {
    var syncstate;
    var cur=getAddressBook(fileName);
    if (cur && cur.directoryProperties.URI!=kPersonalAddressbookURI &&
          cur.directoryProperties.URI!=kCollectedAddressbookURI) {
        var prefName = cur.directoryProperties.prefName;
          //prefName='ldap_2.servers.name' mit name='history', 'pab' oder eigener
        try { syncstate = pref.getBoolPref( prefName+".extensions.addressbookssync" ); }
          catch(e) { syncstate=false; }
        directory.deleteDirectory( cur );
        pref.deleteBranch( prefName );
        pref.deleteBranch( prefName+'.position' );  // sometimes remains!?
        return syncstate;
    }
  } catch (e) { alert('deleteAddressBook('+name+','+fileName+')\n'+e); }
} // deleteAddressBook


//INFO:
  // This attribute servers two purposes
	// depending if the directory is a mail list.
	// If not mail list directories are stored here
	// If so mail list card entries are stored here
//nsISupportsArray nsIAbDirectory.addressLists

	// Get the cards associated with the directory
	// This will return the cards associated with
	// the mailing lists too
//readonly nsIEnumerator nsIAbDirectory.childCards

	// Get the children directories
//readonly nsISimpleEnumerator nsIAbDirectory.childNodes



/*** old version
var delaycreate=false;
var delayedCreateInfo=new Array;
function delayedCreate() {
  var file=delayedCreateInfo.shift();
  var name=delayedCreateInfo.shift();
  var sync=delayedCreateInfo.shift();
  createAddressBook(name, file, sync);
  if (delayedCreateInfo.length>0) setTimeout(delayedCreate, 1000);
}
function replaceAddressBook(mabName, mabFile, abData)
{
    // I found no way to say TB to reread the Addressbookfile. So i delete and recreate
    // the addressbook which does not work for pab(=abook) and cab(=history)
  var syncstate=deleteAddressBook( mabName, mabFile ); // NOP for 'history' und 'pab'!
  writeMabData(mabFile, true, abData);

    // TB may crash, if the just deleted addressbook had the focus in an
    // addressbook view and the create follows to quickly
  if (!delaycreate)
    createAddressBook( mabName, mabFile, syncstate );
  else {
    if (delayedCreateInfo.length==0) setTimeout(delayedCreate, 1000);
    delayedCreateInfo.push(mabFile,mabName,syncstate);
  }
  return;
}
***/


function replaceAddressBook(mabName, mabFile, abData)
{
  tempName='temporary_'+mabName;
/* USE TEMP DIRECTORY
  // Be sure to always use a different name for the tempfiles
  // or strange things may happen with maillists occuring multiple times!
//  var time=(new Date()).getTime().toString();
//  tempName+='_'+time;
//  tempName = tempName.replace( /\W/g, "_" );
USE TEMP DIRECTORY */
  tempFile=tempName+'.mab';
  var file=writeMabData(tempFile, true, abData);

  var cur=getAddressBook(mabFile);  //cur=nsIAbDirectory
  if (!cur) return 'failed to create addressbook';

  //
  // clear target addressbook
  //
  try  {
    var cardsE=cur.childCards;
    var cards = Components.classes["@mozilla.org/supports-array;1"].
      getService(Components.interfaces.nsISupportsArray);
      //convert nsIEnumerator to nsISupportsArray (don't try cur.dropCard()!)
    var goon=1;
      // Loop over the cards
    try { cardsE.first() } catch (e) { goon=0; }
    while (goon) {
      var card = cardsE.currentItem().QueryInterface(Components.interfaces.nsIAbCard);
      cards.AppendElement(card);
      try { cardsE.next() } catch(e) { goon=0; }
    }
    cur.deleteCards(cards);  // delete the cards from the to-be synced addressbook
  
    var lists=cur.childNodes;  //delete lists
    while( lists.hasMoreElements() ) {
      var list = lists.getNext().QueryInterface(Components.interfaces.nsIAbDirectory);
      cur.deleteDirectory(list);
    }
  } catch(e) { alert('clear addressbook: '+e); }

// The cards are not deleted immediately, they are put on a deletedCards Table
// This table can't be purge programmatically, they are automatically purged
// iff more than 50 Cards are in the table and only cards are purged that are
// older than 6 month :-(((
// Therefore I set the LastModifiedDate attribute on the deleted cards to an
// appropriate date (1.1.2000) so they will be deleted later on
  var ab = Components.classes["@mozilla.org/addressbook;1"].
    getService(Components.interfaces.nsIAddressBook);
  var curDB=ab.getAbDatabaseFromURI(cur.directoryProperties.URI);

  try  {
    var count=new Object();
    var dListObj=new Object();
    curDB.getDeletedCardList(count, dListObj);
    if (count.value) {
      var dList=dListObj.value;
      for (var i=0; i<dList.Count(); i++)
      {
        var card = dList.GetElementAt(i).QueryInterface(Components.interfaces.nsIAbCard);
//        386d3570 = 946681200 = 1.Jan.2000, must not be 0!!
        curDB.setCardValue(card, 'LastModifiedDate', '386d3570' , false);
//wow, with this function, i can set my own values!!
      }
    }
  } catch(e) { alert('reset modDate: '+e); }
  curDB.close(true);

/* USE TEMP DIRECTORY
  createAddressBook( tempName, tempFile, false );
  var temp=getAddressBook(tempFile);  //temp=nsIAbDirectory
  var tCards=temp.childCards;
USE TEMP DIRECTORY */

/* USE TEMP DATABASE */
  // open the addressbook database file
  var db=Components.classes["@mozilla.org/addressbook/carddatabase;1"].
    getService(Components.interfaces.nsIAddrDatabase);
  var tdb=db.open(file, true, false);

  // give me the maillists
    // must be moz-abmdbdirectory, not moz-abdirectory
  var temp=Components.classes['@mozilla.org/rdf/resource-factory;1?name=moz-abmdbdirectory']
      .createInstance(Components.interfaces.nsIAbDirectory);
  tdb.getMailingListsFromDB(temp);

  var tCards=tdb.enumerateCards(temp);
/* USE TEMP DATABASE */

  // create maillists
  try  {
    var tMailLists=temp.childNodes;
    while (tMailLists.hasMoreElements()) {
      var tMailList=tMailLists.getNext().QueryInterface(Components.interfaces.nsIAbDirectory);
  
        var nMailList = Components.classes["@mozilla.org/addressbook/directoryproperty;1"].
          createInstance().QueryInterface(Components.interfaces.nsIAbDirectory);
        nMailList.dirName = tMailList.dirName;
        nMailList.listNickName = tMailList.listNickName;
        nMailList.description = tMailList.description;
        cur.addMailList(nMailList);
    }
  }catch(e) { alert('create maillists: '+e); }

  //
  // copy cards from database to mab
  //
// - Thunderbird has no way to copy a maillist! If you drag and drop a maillist
//   inside the addressbook manager the maillist becomes a normal card!!
// - The dirName Property of maillists is unique! Thunderbird refuses to create
//   a maillist with the same name (applies only in the GUI)
// - => How to:
//   create maillists
//   loop over old cards
//     copy card
//     find all maillists, the card is in
//       find the corresponding new maillist
//       insert the copy of the card into these maillists

  try {
    var goon=1;
      // Loop over the cards
    try { tCards.first() } catch (e) { goon=0; }
    while (goon) {
      // need two references to the current card. One for a later use with .equals
      // the other gets modified to be the copied card
      var tCard = tCards.currentItem().QueryInterface(Components.interfaces.nsIAbCard);
      var nCard = tCards.currentItem().QueryInterface(Components.interfaces.nsIAbCard);
if (tCard.isAnEmailAddress || tCard.isASpecialGroup || !tCard.isANormalCard)
alert('unnormal card: '+tCard.displayName+'('+tCard.primaryEmail+') isAnEmailAddress='+tCard.isAnEmailAddress
+' isANormalCard='+tCard.isANormalCard+' isASpecialGroup='+tCard.isASpecialGroup);
            //tCard.isANormalCard is also true for maillist!
      if (!tCard.isMailList) { // only non-maillists
//        var nCard=cur.addCard(tCard);  // add card, afterwards: nCard.equals(tCard) is true !!
        cur.dropCard(nCard, false);  // changes nCard to be the new card!
        //find maillists, the card is in
        var tMailLists=temp.childNodes; //mailLists=nsISimpleEnumerator, Get the children directories
        while (tMailLists.hasMoreElements()) {
          var tMailList=tMailLists.getNext().QueryInterface(Components.interfaces.nsIAbDirectory);
          // loop over cards in maillist
          for (var i=0; i<tMailList.addressLists.Count(); i++)
          {
            var tmlCard = tMailList.addressLists.GetElementAt(i).QueryInterface(Components.interfaces.nsIAbCard);
            if (tCard.equals(tmlCard)) {
              // find new maillist and insert new card
              var nMailLists=cur.childNodes; //mailLists=nsISimpleEnumerator, Get the children directories
              while (nMailLists.hasMoreElements()) {
                var nMailList=nMailLists.getNext().QueryInterface(Components.interfaces.nsIAbDirectory);
                if( tMailList.dirName == nMailList.dirName) {  // got it!
                  nMailList.dropCard(nCard, false);  // insert new card to new maillist
                  break;
                }
              }
              break;
            }
          }
        }
      }
      try { tCards.next() } catch(e) { goon=0; }
    }
  } catch(e) { alert(e); }

/* USE TEMP DIRECTORY
  // remove temporary addressbook
  deleteAddressBook( tempName, tempFile );
USE TEMP DIRECTORY */
/* USE TEMP DATABASE */
  tdb.close(false);

  // delete temporary addressbook file
  // (always throws exception 'access denied' :-((, but seems to be a timing problem! )
//  try { file.remove(true); } catch(e) { alert(e); }
/*
  if (delFiles.length==0) setTimeout(deleteFiles, 1000);
  delFiles.push(file);
*/
/* USE TEMP DATABASE */

  return '';
}

/*
function deleteFiles() {
  while (delFiles.length) {
    var file=delFiles[0];
    try {
      file.remove(false);
      delFiles.shift(); // remove file from list, if remove successfull
    } catch(e) {
//    alert(e);
       break;
    }
  }
  if (delFiles.length) setTimeout(deleteFiles, 1000);  // wait again, if neccessary
}
*/

function getSyncURI(masked)
{
  var syncprotocol;
  var synchost='';
  var syncuser='';
  var syncpassword='';
  var syncpath='';
  try{ syncprotocol = pref.getCharPref( "extensions.addressbookssync.protocol" ); } catch(e){}
  try{ synchost = pref.getCharPref( "extensions.addressbookssync.host" ); } catch(e){}
  try{ syncuser = pref.getCharPref( "extensions.addressbookssync.user" ); } catch(e){}
  try{ syncpassword = get_password(synchost, syncuser); } catch(e){}
  try{ syncpath = pref.getCharPref( "extensions.addressbookssync.path" ); } catch(e){}

  var pwd='';
  if (masked) {
    var pwl=syncpassword.length;
    for(var i=0; i<pwl;i++) pwd += "*";
  } else
    pwd=syncpassword;

  if( ! synchost )
    return "";
  var userpass=syncuser +(pwd?":":"")+pwd;
  if(userpass)
    userpass += "@";
  return syncprotocol + "://" + userpass + synchost + syncpath+'/';
}

function getTimeStamp()
{
    var date = new Date();
    return Math.floor( date.getTime() / 1000 );

} // getTimeStamp


function errorstr(aError)
{
  var error;
  var sError;
  if (aError&0xffff0000) {
    if ((aError>>>16)==0x804b)
      sError=aError&0xffff
    else
      sError='0x'+aError.toString(16);
  } else
    sError=aError;

  switch(aError){
  // internal errors
    case -1:
      error='unexpected initialization error';
      break;
    case -2:
      error='local mab file read error';  //access denied (also gives unhandled exception), does not exist
      break;
  //external errors (server error, windows error)
    case 0x804b000d:
      error='protocol not available';
      break;
    case 0x804b001e:
      error='host not found';
      break;
    case 0x80004005:
      error=sError+': illegal path';  //??? (with FTP, if target directory does not exist or else)
      break;
    case 405:
      error=sError+': method not allowed';  //method is something like PUT,POST or GET
      break;
    case 404:
      error=sError+': not found';
      break;
    case 403:
      error=sError+': permission denied';
      break;
    case 0x804b0015:
    case 401:
      error=sError+': illegal credentials';
      break;
    case 0x804b000e:
      error='timeout';
      break;
    case 0x804b002a:
      error='proxy not found';
      break;
    case 0x804b0048:
      error='proxy error';
      break;
    case 0x804b0016:
      error=strings.getString('nofile');
      break;
    case 0:
    case 201:
      error=strings.getString('ok');
      break;
    default:
      error='Error '+sError;
      break;
  }
  return error;
}

function addMenu()
{
  try{
    var menu=document.getElementById("tasksMenu");
    var popup=menu.firstChild;
    var sep=document.getElementById("prefSep");
    var strings = document.getElementById("strings");
    var label=strings.getString('menuname'); //"Addressbooks Synchronizer";
    var accesskey=strings.getString('menushortcut');
  }catch(e){alert(e); return;}

  var  element = document.createElementNS("http://www.mozilla.org/keymaster/gatekeeper/there.is.only.xul", "menuitem");
  element.setAttribute("id", "gg-addressbookSync");
  element.setAttribute("label", label);
  element.setAttribute("accesskey", accesskey);
  element.setAttribute("oncommand", "showOptions();");
  popup.insertBefore(element,sep);
}


function statustxt(aStatus, newline, doalert)
{
  var usespopup=window.location=='chrome://addressbookssync/content/download.xul'
    || window.location=='chrome://addressbookssync/content/upload.xul';
  if (usespopup) {
    if (newline) document.getElementById("status").value += "\n";
    document.getElementById("status").value += aStatus;
  } else {
    var maindoc = window.location!='chrome://messenger/content/messenger.xul'?window.opener.document:document;
    if (newline) maindoc.getElementById('statusText').label=aStatus
    else maindoc.getElementById('statusText').label+=aStatus;
    if (doalert)
      alert(maindoc.getElementById('statusText').label);
  }
}

function showOptions()
{
  window.open("chrome://addressbookssync/content/options.xul","_blank","chrome,resizable,titlebar=yes");
}
function showPopup(popup, usepopup)
{
  if (syncedAddressbooks().length==0) return;

  var hidepopups;
  if (usepopup) hidepopup=false;
  else
    try{ hidepopups = pref.getBoolPref( "extensions.addressbookssync.hidepopups" ); }
      catch(e){ alert(e); hidepopups=false; }
  if (hidepopups) {
    if (popup=='download') download();
    else upload();
  } else {
    var x=Math.floor(window.outerWidth/2+window.screenX);
    var y=Math.floor(window.outerHeight/2+window.screenY);
    var w=window.open("chrome://addressbookssync/content/"+popup+".xul","_blank",'chrome,resizable,titlebar=no');
    w.screenX=x;
    w.screenY=y;
  }
}

function onUnload()
{
  if (window.location=='chrome://messenger/content/messenger.xul') {
    var windowsEnum = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService()
                     .QueryInterface(Components.interfaces.nsIWindowMediator)
                     .getEnumerator("mail:3pane");
    if (!windowsEnum.hasMoreElements()) { //no more windows, this was the last one! Sync!
      var autoupload;
      try{ autoupload = pref.getBoolPref( "extensions.addressbookssync.autoupload" ); }
        catch(e){ autoupload = false; }
      if (autoupload) showPopup('upload', true);
    }
  }
}

function onLoad()
{
  if (window.location=='chrome://messenger/content/messenger.xul') {
    var windowsEnum = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService()
                     .QueryInterface(Components.interfaces.nsIWindowMediator)
                     .getEnumerator("mail:3pane");
    var w=windowsEnum.getNext();  // there must be one (the current)!
    if (!windowsEnum.hasMoreElements()) { //no more windows, this is the first! Sync!
      var autodownload;
      try{ autodownload = pref.getBoolPref( "extensions.addressbookssync.autodownload" ); }
        catch(e){ autodownload = false; }
      if (autodownload) showPopup('download', false);
      addMenu();  // add addressbookssync to tools-menu
    }
  }
}

//Event Listeners
function delayLoad() {
    //without the delay of some (one) second and we are using a master
    //password, the window to enter the masterpassword popups twice!
  setTimeout(onLoad, 1000);
}
window.addEventListener("load", delayLoad, false);
window.addEventListener("unload", onUnload, false);
